<?php /**
 * @file
 * Contains \Drupal\cleantalk\EventSubscriber\BootSubscriber.
 */

namespace Drupal\cleantalk\EventSubscriber;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Drupal\cleantalk\CleanTalkSFW;

class BootSubscriber implements HttpKernelInterface  {

  /**
   * The wrapped HTTP kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * Creates a HTTP middleware handler.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $kernel
   *   The HTTP kernel.
   */
  public function __construct(HttpKernelInterface $kernel) {
    $this->httpKernel = $kernel;
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, $type = self::MASTER_REQUEST, $catch = TRUE) {
    // Custom logic goes here.
     $is_sfw = \Drupal::config('cleantalk.settings')->get('cleantalk_sfw');
    if($is_sfw == 1)
    {   
      $ct_key = \Drupal::config('cleantalk.settings')->get('cleantalk_authkey');
      if($ct_key != ''){
      
      $ct_sfw_last_logs_sent =\Drupal::config('cleantalk.settings')->get('cleantalk_sfw_last_logs_sent');
      if($ct_sfw_last_logs_sent && (time() - $ct_sfw_last_logs_sent > 3600)){
        CleanTalkSFW::send_logs($ct_key);
        \Drupal::service('config.factory')->getEditable('cleantalk.settings')->set('cleantalk_sfw_last_logs_sent', time());        
      }
      
      $ct_sfw_last_updated = \Drupal::config('cleantalk.settings')->get('cleantalk_sfw_last_updated');
      if($ct_sfw_last_updated && (time()-$ct_sfw_last_updated > 86400))
      {
        CleanTalkSFW::sfw_update($ct_key);
        \Drupal::configFactory()->getEditable('cleantalk.settings')->set('cleantalk_sfw_last_updated', time());
      }
        
      
            $is_sfw_check = true;
            $ip = $this->CleantalkGetIP();
      
            $ip = array_unique($ip);
            foreach($ip as $key => $value){
        if(isset($_COOKIE['ct_sfw_pass_key']) && $_COOKIE['ct_sfw_pass_key'] == md5($value . $ct_key)){
          $is_sfw_check=false;
          if(isset($_COOKIE['ct_sfw_passed'])){
            @setcookie ('ct_sfw_passed'); //Deleting cookie
            CleanTalkSFW::sfw_update_logs($value, 'passed');
          }
        }
      } unset($key, $value);
      
        if($is_sfw_check){
          $sfw = new CleanTalkSFW();
          $sfw->cleantalk_get_real_ip();
          $sfw->check_ip();
          if($sfw->result){
            CleanTalkSFW::sfw_update_logs($sfw->blocked_ip, 'blocked');
            $sfw->sfw_die();
          }
      }
        }
    }
    $ct_ccf = \Drupal::config('cleantalk.settings')->get('cleantalk_ccf');
    if (sizeof($_POST) > 0 && !isset($_POST['form_build_id']) && !isset($_POST['form_id']) && $ct_ccf == 1) //Custom Contact forms
    {
      $sender_email = '';
      $sender_nickname = '';
      $subject = '';
      $message = '';
      $contact_form = TRUE;
      $this->get_fields_any($sender_email, $message, $sender_nickname, $subject, $contact_form, $_POST);

      if ($sender_email != '') {
        $ct_authkey = \Drupal::config('cleantalk.settings')->get('cleantalk_authkey');

        if (!isset($_COOKIE['ct_checkjs'])) {
          $checkjs = NULL;
        }
        elseif ($_COOKIE['ct_checkjs'] == _cleantalk_get_checkjs_value()) {
          $checkjs = 1;
        }
        else {
          $checkjs = 0;
        }

        $data = [
          'type' => 'comment',
          'auth_key' => $ct_authkey,
          'method_name' => 'check_message',
          'message' => $message,
          'example' => NULL,
          'agent' => CLEANTALK_USER_AGENT,
          'sender_ip' => $_SERVER['REMOTE_ADDR'],
          'sender_email' => $sender_email,
          'sender_nickname' => $sender_nickname,
          'message_title' => $subject,
          'message_body' => '',
          'example_title' => '',
          'example_body' => '',
          'example_comments' => '',
          'comment_body' => '',
          'js_on' => $checkjs,
        ];

        $result = _cleantalk_check_spam($data);
        if ($result['allow'] === 0) {
          drupal_set_message($result['ct_result_comment'], 'error');
        }
      }
    }
    if (isset($_POST['mergevars'])) 
    {
      $sender_email = NULL;
      $message = '';

      foreach ($_POST['mergevars'] as $key => $value) {
        if ($sender_email === NULL && preg_match("/^\S+@\S+\.\S+$/", $value)) {
          $sender_email = $value;
        }
        else {
          $message .= "$value\n";
        }
      }

      if ($sender_email != NULL) {

        $url = 'http://moderate.cleantalk.org/api2.0';

        $ct_authkey = \Drupal::config('cleantalk.settings')->get('cleantalk_authkey');

        if (!isset($_COOKIE['ct_checkjs'])) {
          $checkjs = NULL;
        }
        elseif ($_COOKIE['ct_checkjs'] == _cleantalk_get_checkjs_value()) {
          $checkjs = 1;
        }
        else {
          $checkjs = 0;
        }

        $data = [
          'type' => 'comment',
          'auth_key' => $ct_authkey,
          'method_name' => 'check_message',
          'message' => $message,
          'example' => NULL,
          'agent' => CLEANTALK_USER_AGENT,
          'sender_ip' => $_SERVER['REMOTE_ADDR'],
          'sender_email' => $sender_email,
          'sender_nickname' => '',
          'message_title' => '',
          'message_body' => '',
          'example_title' => '',
          'example_body' => '',
          'example_comments' => '',
          'comment_body' => '',
          'js_on' => $checkjs,
        ];

        $result = _cleantalk_check_spam($data);
        if ($result['allow'] === 0) {
          drupal_set_message($result['ct_result_comment'], 'error');
          $_POST['mergevars']['EMAIL'] = '';
        }
      }
    }   
    return $this->httpKernel->handle($request, $type, $catch);
  }
  function CleantalkGetIP()
  {
    $result=Array();
    if ( function_exists( 'apache_request_headers' ) )
    {
      $headers = apache_request_headers();
    }
    else
    {
      $headers = $_SERVER;
    }
    if ( array_key_exists( 'X-Forwarded-For', $headers ) )
    {
      $the_ip=explode(",", trim($headers['X-Forwarded-For']));
      $result[] = trim($the_ip[0]);
    }
    if ( array_key_exists( 'HTTP_X_FORWARDED_FOR', $headers ))
    {
      $the_ip=explode(",", trim($headers['HTTP_X_FORWARDED_FOR']));
      $result[] = trim($the_ip[0]);
    }
    $result[] = filter_var( $_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 );

    if(isset($_GET['sfw_test_ip']))
    {
      $result[]=$_GET['sfw_test_ip'];
    }
    return $result;
  }
  function get_fields_any(&$email,&$message,&$nickname,&$subject, $arr) {
      foreach($arr as $key=>$value) {
          if(!is_array($value)&&!is_object($value)){
              if ($email === '' && @preg_match("/^\S+@\S+\.\S+$/", $value)){
                  $email = $value;
              }
              else if ($nickname === '' && $this->get_data_from_submit($value, 'name')) {
                  $nickname = $value;
              }
              else if ($subject === '' && $this->get_data_from_submit($value, 'subject')) {
                  $subject = $value;
              }
              else {
                  @$message.="$value\n";
              }
          }
          else if(!is_object($value)) {
              $this->get_fields_any($email, $message, $nickname, $subject, $value);
          }
      }
  }
  /**
   * Inner function - Finds and returns pattern in string
   * @return null|bool
   */
  function get_data_from_submit($value = null, $field_name = null) 
  {
      if (!$value || !$field_name || !is_string($value)) {
          return false;
      }
      if (preg_match("/[a-z0-9_\-]*" . $field_name. "[a-z0-9_\-]*$/", $value)) {
          return true;
      }
  }
}
